<?php
/*
Plugin Name: elegantPURL Lite
Description: A lite PURL plugin for WordPress meant to be so easy anyone can use it.
Version: 0.92
Author: Chris Lema (cflema@gmail.com) and Justin Kussow (jdkussow@gmail.com)
License: GPLv2
*/

/* Directories and URLs */
define( 'EPURL_URL', plugin_dir_url(__FILE__) );
define( 'EPURL_DIR', plugin_dir_path(__FILE__) );

class elegantpurl {
    static $instance = false;

    function __construct() {
        register_activation_hook( __FILE__, array( &$this, 'install' ) );
                
        add_action( 'rewrite_rules_array', array( &$this, 'add_rewrite_rules' ) );
        add_filter( 'query_vars', array( &$this, 'insert_query_vars' ) );
        add_action( 'init', array( &$this, 'handle_init' ) );
        add_action( 'admin_menu', array( &$this, 'add_admin_menu' ) );
        add_action( 'wp_print_scripts', array(&$this, 'print_scripts') );
        add_action( 'delete_post', array( &$this, 'clean_after_post_delete' ));
        
        if ( !class_exists( 'epurl_js' ) ) 
            include_once ( EPURL_DIR . 'epurl-js.php' );
        
        if ( !class_exists( 'epurl_shortcodes' ) ) 
            include_once ( EPURL_DIR . 'epurl-shortcodes.php' ); 
        epurl_shortcodes::reload_all();
        
        if ( !class_exists( 'epurl_campaign_ui' ) ) 
            include_once ( EPURL_DIR . 'epurl-campaign-ui.php' ); 
    }
    
    function install() { }
    
    function reset() {
        $epurl_reset = true;
        include ( EPURL_DIR . 'uninstall.php' );
        unset($epurl_reset);
    }
    
    function handle_init() {
        self::add_custom_post_type();
        self::add_epurl_to_mce();
    }
    
    function add_custom_post_type() {
        register_post_type(
            'epurl_campaign',
            array(
                'labels' => array(
                    'name' => __( 'Campaigns' ),
                    'singular_name' => __( 'Campaign' ),
                    'add_new' => _x('Add New', 'campaign'),
                    'add_new_item' => __('Add New Campaign'),
                    'edit_item' => __('Edit Campaign'),
                    'new_item' => __('New Campaign'),
                    'all_items' => __('All Campaigns'),
                    'view_item' => __('View Campaign'),
                    'search_items' => __('Search Campaigns'),
                    'not_found' =>  __('No campaigns found'),
                    'not_found_in_trash' => __('No campaigns found in Trash')
                    ),
                'public' => true,
                'publicly_queryable' => false,
                'exclude_from_search' => true,
                'supports' => array( 'title' )
            )
	);
        
        register_post_type( 
            'epurl_member',
            array(
                'labels' => array(
                    'name' => __( 'Members' ),
                    'singular_name' => __( 'Member' ),
                    'search_items' => __('Search Members'),
                    'not_found' =>  __('No members found'),
                    'not_found_in_trash' => __('No members found in Trash')
                    ),
                'public' => true,
                'exclude_from_search' => true,
                'show_in_menu' => false,
                'supports' => array( 'title' )
            )
	);
    }
    
    function add_epurl_to_mce() {
        if ( !current_user_can('edit_posts') && !current_user_can('edit_pages') ) {
            return;
        }
        if ( get_user_option('rich_editing') == 'true' ) {
            add_filter( 'mce_buttons', array( &$this, 'filter_mce_buttons' ) );
            add_filter( 'mce_external_plugins', array( &$this, 'filter_mce_external_plugins' ) );
        }
    }
    
    function filter_mce_buttons( $buttons ) {
        array_push( $buttons, '|', 'epurl_shortcode_button' );
        return $buttons;
    }
    
    function filter_mce_external_plugins( $plugin_array ) {
        $plugin_array['epurl'] = EPURL_URL . 'includes/epurlmce.js.php?rand=' . time();
        return $plugin_array;
    }
    
    function add_rewrite_rules( $rules ) {
        global $wpdb;
        
        // get the campaign posts and their landing pages from the db
        $landing_pages = $wpdb->get_results( $wpdb->prepare(
                "SELECT p.post_name as campaign_name, p2.ID as landing_page_id, p2.post_name as landing_page, p2.post_type as landing_page_type
                 FROM $wpdb->posts p
                 JOIN $wpdb->postmeta pm ON p.ID = pm.post_id
                 JOIN $wpdb->posts p2 on pm.meta_value = p2.ID
                 WHERE p.post_status in ('private', 'publish') AND p.post_type = 'epurl_campaign' AND pm.meta_key = 'epurl_landing_page'"
                ));

        // build an array of new rules to add to WP's rewrite rules for our landing pages
        $newrules = array();
        $campaigns_added = array();
        foreach ( $landing_pages as $landing_page ) {
            $pre = 'pagename=' . $landing_page->landing_page;
            $newrules[$landing_page->campaign_name . '/(\w*)$'] = 'index.php?' . $pre . '&campaign=' . $landing_page->campaign_name . '&user=$matches[1]';
            if ( !in_array($landing_page->campaign_name, $campaigns_added) ) {
                $newrules[$landing_page->campaign_name] = 'index.php?' . $pre . '&campaign=' . $landing_page->campaign_name;
                $campaigns_added[] = $landing_page->campaign_name;
            }
        }
        if ( count($newrules) > 0 ) 
            return $newrules + $rules;
        
        return $rules;
    }
    
    function flush_rewrite_rules() {
        global $wp_rewrite;
        $wp_rewrite->flush_rules();
    }
    
    function insert_query_vars( $vars ) {
        array_push( $vars, 'user' );
        array_push( $vars, 'campaign' );
        return $vars;
    }
    
    function get_option( $opt ) {
        $options = get_option( 'plugin_epurl_settings' );
        return htmlspecialchars_decode( stripslashes( $options[$opt] ) );
    }
    
    function add_admin_menu() {
        if ( !class_exists( 'epurl_settings' ) ) 
            require_once ( EPURL_DIR . 'epurl-settings.php' );
        add_options_page( __('ePURL Lite'), __('ePURL Lite'), 'manage_options', 'epurl', array( 'epurl_settings', 'show_settings_page' ) );
    }
    
    function print_scripts() {
        wp_register_style( 'jqueryui', EPURL_URL . 'includes/ui-darkness/jquery-ui-1.8.20.custom.css' );
        wp_enqueue_style( 'jqueryui' );
        
        wp_register_style( 'epurl', EPURL_URL . 'includes/epurl.css' );
        wp_enqueue_style( 'epurl' );
         
        epurl_js::add_scripts();
    }
    
    // log a message to a log file with a timestamp and move to new line
    function log_it( $msg ) {
        $msg =  '[' . date( "Y-m-d H:i:s" ) . '] ' . $msg . "\r\n";
        $error = "";
        
        $log_dir = EPURL_DIR . 'logs';
        if ( !is_dir($log_dir) )
            mkdir( $log_dir );
        
        $filename = $log_dir . '/elegantpurl.log';
        if ( empty( $error ) && !$handle = fopen( $filename, 'a+' ) ) {
            $error = "Cannot open file ($filename)";
        }
        if ( empty( $error ) && fwrite( $handle, $msg ) === FALSE ) {
            $error = "Cannot write to file ($filename)";
        }
        fclose( $handle );
    }
    
    function get_member_id( $campaign_name, $landing_page ) {
        global $wpdb;
        
        $sql = $wpdb->prepare("
                SELECT p.ID
                FROM $wpdb->posts p
                JOIN $wpdb->postmeta pm ON p.ID = pm.post_id
                JOIN $wpdb->posts pp on p.post_parent = pp.ID
                WHERE pp.post_name = %s AND pm.meta_key = 'epurl_landing_page' AND pm.meta_value = %s
            ", $campaign_name, $landing_page);
        $id = $wpdb->get_var( $sql );
        
        if ( !$id ) {
            // a member was not found, to trick the shortcodes into grabbing the default value from
            // the campaign return the campaign id
            $id = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_name = %s", $campaign_name ) );    
        }
        
        return $id;
    }
    
    function get_member_field_data( $id, $field ) {
        $val = '';
        $p = get_post( $id );
        if ( $p ) {
            // fields are under the same post meta key in members and campaigns
            $fields = get_post_meta( $id, 'epurl_fields', true );
            if ( $fields && $fields[$field] ) {
                if ( $p->post_type == 'epurl_member' ) {
                    $val = $fields[$field]['value'];
                    
                    if ( !$val ) {
                        // let's try to get the default from the campaign since the
                        // member's value was empty
                        $val = get_member_field_data( $p->post_parent, $field );
                    }
                } else if ( $p->post_type == 'epurl_campaign' ) {
                    $val = $fields[$field]['default_value'];
                }
            }
        }
        return $val;
    }
    
    function clean_after_post_delete( $id ) {
        global $wpdb;
        
        $post = get_post( $id );
        
        // kill the campaign members' post meta data
        $wpdb->query( $wpdb->prepare("
                    DELETE pm
                    FROM $wpdb->postmeta pm
                    JOIN $wpdb->posts p ON pm.post_id = p.ID
                    WHERE p.post_type = 'epurl_member' AND p.post_parent = %d
                ", $id) );

        // kill the campaign member posts
        $wpdb->query( $wpdb->prepare("
                    DELETE FROM $wpdb->posts
                    WHERE post_type = 'epurl_member' AND p.post_parent = %d
                ", $id) );
    }
    
    function validate_landing_page( $campaignId, $landingPage ) {
        global $wpdb;
        $metaId = $wpdb->get_var( $wpdb->prepare("
            SELECT pm.meta_id
            FROM $wpdb->posts p
            JOIN $wpdb->postmeta pm ON p.ID = pm.post_id
            WHERE p.post_parent = %d 
                AND pm.meta_key = 'epurl_landing_page'
                AND pm.meta_value = %s;
            ", $campaignId, $landingPage ));
         return !$metaId;
    }
    
    function bump_request_timeout() {
        return 60;
    }
    
    public static function get_instance() {
        if ( !self::$instance ) {
            self::$instance = new self;
        }
        return self::$instance;
    }
}

elegantpurl::get_instance();
?>
